package com.gyee.redis2taos.service;

import com.gyee.redis2taos.config.TaosConfig;
import com.gyee.redis2taos.timeseries.BasicTsPoint;
import com.gyee.redis2taos.timeseries.TsDataType;
import com.gyee.redis2taos.timeseries.TsPoint;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TaosService {

    @Resource
    private TaosConfig taosConfig;
    @Resource
    private FileService fileService;
    @Resource
    private RedisService redisService;

    public boolean createDataBase(int keepday) throws Exception {
        log.info("创建taos库!");
        Statement st = taosConfig.getCreateCon().createStatement();
        String createSTableSql = "create database if not exists " + taosConfig.getDbName() + " cachemodel 'last_row' wal_retention_period 1800 keep " + keepday;
        int i = st.executeUpdate(createSTableSql);
        //taosConfig.closeCon();
        if (i > 0) {
            return true;
        } else {
            return false;
        }
    }


    public boolean createStables() throws Exception {
        log.info("创建taos超级表!");
        Statement st = taosConfig.getInstance().createStatement();
        String sqlai = "create stable if not exists " + taosConfig.getDbName() + "." + taosConfig.getAIStableName() + " (ts timestamp,val double) " +
                "tags (description nchar(100),station binary(20),category binary(20),facility binary(20),uniformcode binary(20),rate float,remark binary(30));";
        int a = st.executeUpdate(sqlai);
        String sqladi = "create stable if not exists " + taosConfig.getDbName() + "." + taosConfig.getDIStableName() + " (ts timestamp,val bool) " +
                "tags (description nchar(100),station binary(20),category binary(20),facility binary(20),uniformcode binary(20),rate float,remark binary(30));";
        int d = st.executeUpdate(sqladi);
        if ((a + d) > 0) {
            return true;
        } else {
            return false;
        }
    }

    public List<String> showDataBase() throws Exception {
        Statement st = taosConfig.getInstance().createStatement();
        String showSTableSql = "SHOW DATABASES;";
        ResultSet resultSet = st.executeQuery(showSTableSql);
        List<String> ss = new ArrayList<>();
        int i = 0;
        while (resultSet.next()) {
            ss.add(resultSet.getString(i));
            i++;
        }
        return ss;
    }

    public int createTables() throws Exception {
        log.info("创建taos表!");
        Statement st = taosConfig.getInstance().createStatement();
        List<StringBuilder> excelTaos = fileService.getExcelTaos();
        int i = 0;
        for (StringBuilder excelTao : excelTaos) {
            int i1 = st.executeUpdate(excelTao.toString() + ";");
            i += i1;
        }
        log.info("taos表创建完成！");
        return i;
    }

    public int createTables2() throws Exception {
        log.info("创建taos表!");
        Statement st = taosConfig.getInstance().createStatement();

        Set<String> keys = redisService.getKeys();
        if(keys.size()==0) return 0;
        log.info("读取rediskeys成功！！！" + keys.size() + "个点。");
        //按类型分点
        Map<TsDataType, List<BasicTsPoint>> di = keys.stream().map(ss -> {
            if (ss.contains("DI") || ss.contains("di")) {
                return new BasicTsPoint(ss, TsDataType.BOOLEAN);
            } else {
                return new BasicTsPoint(ss, TsDataType.DOUBLE);
            }
        }).collect(Collectors.groupingBy(TsPoint::getTsDataType));
        //di点
        List<String> boolCollect = di.get(TsDataType.BOOLEAN).stream().map(k1 -> k1.getId()).collect(Collectors.toList());
        //ai点
        List<String> douCollect = di.get(TsDataType.DOUBLE).stream().map(k1 -> k1.getId()).collect(Collectors.toList());

        int k = 0;
        for (String s : boolCollect) {
            st.executeUpdate(getDiSql(s));
            log.info(String.valueOf(++k));
        }
        for (String s : douCollect) {
            st.executeUpdate(getAiSql(s));
            log.info(String.valueOf(++k));
        }
        log.info("taos表创建完成！");
        return 1;
    }

    private String getAiSql(String s) {
        return "CREATE TABLE IF NOT EXISTS "+taosConfig.getDbName()+"."+s+" USING "+taosConfig.getDbName()
                +"."+taosConfig.getAIStableName()+" TAGS('','','','','',1,'');";
    }

    private String getDiSql(String s) {
        return "CREATE TABLE IF NOT EXISTS "+taosConfig.getDbName()+"."+s+" USING "+taosConfig.getDbName()
                +"."+taosConfig.getDIStableName()+" TAGS('','','','','',1,'');";
    }

    public List<String> deleteTables() throws Exception {
        Statement st = taosConfig.getInstance().createStatement();
        String showSTableSql = "SHOW DATABASES;";
        ResultSet resultSet = st.executeQuery(showSTableSql);
        List<String> ss = new ArrayList<>();
        int i = 0;
        while (resultSet.next()) {
            ss.add(resultSet.getString(i));
            i++;
        }
        return ss;
    }

    public boolean updateTables(int keepday) throws Exception {
        log.info("创建taos库!");
        Statement st = taosConfig.getInstance().createStatement();
        String createSTableSql = "create database if not exists " + taosConfig.getDbName() + " replica 1 quorum 1 days 1 keep " + keepday + " cache 16 blocks 6" +
                " minrows 100 maxrows 4096 wal 1 fsync 3000 comp 2 cachelast 0 precision 'ms' update 1";
        int i = st.executeUpdate(createSTableSql);
        if (i > 0) {
            return true;
        } else {
            return false;
        }
    }

    public List<String> selectTables() throws Exception {
        Statement st = taosConfig.getInstance().createStatement();
        String showSTableSql = "SHOW DATABASES;";
        ResultSet resultSet = st.executeQuery(showSTableSql);
        List<String> ss = new ArrayList<>();
        int i = 0;
        while (resultSet.next()) {
            ss.add(resultSet.getString(i));
            i++;
        }
        return ss;
    }
}
